@@ -54,9 +54,10 @@ def mimo(
     wwg_south=None,
     wwg_north=None,
     taper=0,
-    init_geometry_io=[],
+    init_geometry=None,
     layer=WG,
     design_layer=DESIGN,
+    centered=False,
     **kwargs,
 ):
     if not length_wg:
@@ -64,7 +65,18 @@ def mimo(
     design = gf.Component()
     c = gf.Component(**kwargs)
 
-    p = [(0, 0), (l, 0), (l, w), (0, w)]
+    if centered:
+        xmin = -l / 2
+        ymin = -w / 2
+        xmax = l / 2
+        ymax = w / 2
+        p = [(-l / 2, -w / 2), (l / 2, -w / 2), (l / 2, w / 2), (-l / 2, w / 2)]
+    else:
+        xmin = 0
+        ymin = 0
+        xmax = l
+        ymax = w
+        p = [(0, 0), (l, 0), (l, w), (0, w)]
     design.add_polygon(p, layer=design_layer)
     # c.add_polygon(p,                       layer=layer)
 
@@ -84,8 +96,8 @@ def mimo(
     nports = 0
     for i, x, y, ds, wwgs, orientation in zip(
         range(4),
-        [0, 0, l, l],
-        [0, w, w, 0],
+        [xmin, xmin, xmax, xmax],
+        [ymin, ymax, ymax, ymin],
         port_pos_sides,
         wwg_sides,
         [180, 90, 0, -90],
@@ -129,25 +141,36 @@ def mimo(
     # p.extend([a+s*d * (2*s*n+(1-2*s)*n1) for s in np.linspace(0, .5, 50)])
     # p.extend(reversed([b+s*d * (-2*s*n+(1-2*s)*n2)
     #          for s in np.linspace(0, .5, 50)]))
-    for i, j in init_geometry_io:
-        pi = design.ports[f"o{i}"]
-        pj = design.ports[f"o{j}"]
-        a = np.array(pi.center)
-        b = np.array(pj.center)
-        n1 = -np.array(
-            [cos(np.radians(pi.orientation)), sin(np.radians(pi.orientation))]
-        )
-        n2 = -np.array(
-            [cos(np.radians(pj.orientation)), sin(np.radians(pj.orientation))]
-        )
-        v = b - a
-        d = np.linalg.norm(v)
-        n = v / d
-        l = [a, a + 0.5 * d * n1, b + 0.5 * d * n2, b]
+    if type(init_geometry) is list:
+        for i, j in init_geometry:
+            pi = design.ports[f"o{i}"]
+            pj = design.ports[f"o{j}"]
+            a = np.array(pi.center)
+            b = np.array(pj.center)
+            n1 = -np.array(
+                [cos(np.radians(pi.orientation)), sin(np.radians(pi.orientation))]
+            )
+            n2 = -np.array(
+                [cos(np.radians(pj.orientation)), sin(np.radians(pj.orientation))]
+            )
+            v = b - a
+            d = np.linalg.norm(v)
+            n = v / d
+            l = [a, a + 0.5 * d * n1, b + 0.5 * d * n2, b]
 
-        c << gf.components.bends.bezier(
-            [x.tolist() for x in l], allow_min_radius_violation=True, width=pi.width
-        )
-    if not init_geometry_io:
+            c << gf.components.bends.bezier(
+                [x.tolist() for x in l], allow_min_radius_violation=True, width=pi.width
+            )
+    if init_geometry == 1:
         c.add_polygon(p, layer=layer)
+    p = [
+        {
+            "name": v.name,
+            "center": v.center,
+            "width": v.width,
+            "orientation": v.orientation,
+        }
+        for v in design.ports
+    ]
+    c.info["ports"] = p
     return c
