@@ -98,15 +98,25 @@ def make_design_prob(
     return prob
 
 
-def Target(key, value, weight, type="T", frequency=None, wl_1_f=None, wavelength=None):
+def Target(
+    key,
+    target,
+    weight,
+    param="T",
+    frequency=None,
+    wl_1_f=None,
+    wavelength=None,
+    func="abs",
+):
     if frequency is not None:
         wavelength = wl_1_f / frequency
     return {
-        "type": type,
-        "key": key,
+        "param": param,
+        "key": str(key),
         "wavelength": wavelength,
-        "value": value,
+        "target": target,
         "weight": weight,
+        "func": func,
     }
 
 
