@@ -21,7 +21,7 @@ from gdsfactory.generic_tech import LAYER_STACK, LAYER
 def setup(
     path,
     study,
-    center_wavelength,
+    wavelength,
     wavelengths,
     wl_1_f=None,
     bbox=None,
@@ -59,9 +59,9 @@ def setup(
     T=None,
     field_decay_threshold=None,
     path_length_multiple=None,
-    relpml=1,
+    relative_pml_depths=1,
     relcourant=0.9,
-    hasmetal=None,
+    hasPEC=None,
     keys=None,
     ordering="frequency",
     #
@@ -70,13 +70,9 @@ def setup(
     force=False,
     verbose=False,
     pixel_size=0.01,
+    secret=None,
+    info=None,
 ):
-    print(z)
-    if ports is None:
-        ports = {}
-    if modes is None:
-        modes = []
-
     # if force:
     #     shutil.rmtree(path, ignore_errors=True)
     # elif os.path.exists(path):
@@ -85,7 +81,9 @@ def setup(
     #     )
 
     os.makedirs(path, exist_ok=True)
-
+    if info is None:
+        info = {}
+    json.dump(info, open(os.path.join(path, "info.json"), "w"), indent=4)
     if approx_2D_mode:
         N = 2
     else:
@@ -97,7 +95,7 @@ def setup(
 
     prob = {
         "nres": nres,
-        "center_wavelength": center_wavelength,
+        "wavelength": wavelength,
         "wavelengths": wavelengths,
         "name": name,
         "path": path,
@@ -116,11 +114,12 @@ def setup(
         "geometry_slices": geometry_slices,
         "boundaries": boundaries[0:N],
         "approx_2D_mode": approx_2D_mode,
-        "relpml": relpml,
+        "relative_pml_depths": relative_pml_depths,
         "relcourant": relcourant,
-        "hasmetal": hasmetal,
+        "hasPEC": hasPEC,
         "ordering": ordering,
         "pixel_size": pixel_size,
+        "secret": secret,
     }
 
     prob["class"] = "pic"
@@ -138,20 +137,28 @@ def setup(
         0
     else:
         c = component
+        if ports is None:
+            ports = []
 
         if bbox is None:
             bbox = c.bbox_np()
         l, w = bbox[1] - bbox[0]
-        for k, v in ports.items():
+        for v in ports:
             if "layer" in v:
                 0
             else:
-                v["center"] = (sum(bbox) / 2).tolist() + [v["z"]]
-                v["start"] = [-l / 2, -w / 2]
-                v["stop"] = [l / 2, w / 2]
-                if v["sign"] > 0:
+                if "z" in v:
+                    v["center"] = (sum(bbox) / 2).tolist() + [v["z"]]
+                x, y = v["center"][:2]
+                if "length" not in v:
+                    v["length"] = l
+                if "width" not in v:
+                    v["width"] = w
+                v["start"] = [-v["length"] / 2, -v["width"] / 2]
+                v["stop"] = [+v["length"] / 2, +v["width"] / 2]
+                if v["direction"] == "+z":
                     frame = [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
-                else:
+                elif v["direction"] == "-z":
                     frame = [[-1, 0, 0], [0, 1, 0], [0, 0, -1]]
                 v["frame"] = frame
                 v["normal"] = frame[2]
@@ -159,6 +166,7 @@ def setup(
 
         for p in c.get_ports_list(prefix="o"):
             v = {
+                "name": p.name,
                 "center": (np.array(p.center) / 1e0).tolist(),
                 "normal": [
                     cos(p.orientation / 180 * pi),
@@ -173,7 +181,7 @@ def setup(
             z, n = [0, 0, 1], [*v["normal"], 0]
             t = np.cross(z, n).tolist()
             v["frame"] = [t, z, n]
-            ports[p.name] = v
+            ports.append(v)
 
         prob["ports"] = ports
 
@@ -222,6 +230,34 @@ def setup(
 
         layers = set(c.layers) - set(exclude_layers)
 
+        if modes is None:
+            modes = [
+                {
+                    "wavelength_range": [wavelengths[0], wavelengths[-1]],
+                    "nmodes": 1,
+                }
+            ]
+        for mode in modes:
+            # if "wavelength" in mode:
+            #     assert "frequency" not in mode
+            #     mode["frequency"] = wl_1_f / mode["wavelength"]
+            #     if "bandwidth" in mode:
+            #         mode["bandwidth"] *= mode["frequency"] / mode["wavelength"]
+            # if "frequency" in mode:
+            #     assert "wavelength" not in mode
+            #     mode["wavelength"] = wl_1_f / mode["frequency"]
+            #     if "bandwidth" in mode:
+            #         mode["bandwidth"] *= mode["wavelength"] / mode["frequency"]
+            if "frequency_range" in mode:
+                mode["wavelength_range"] = [
+                    wl_1_f / mode["frequency_range"][1],
+                    wl_1_f / mode["frequency_range"][0],
+                ]
+            if "wavelength_range" not in mode:
+                mode["wavelength_range"] = [wavelengths[0], wavelengths[-1]]
+            if "ports" not in mode:
+                mode["ports"] = None
+
         MODES = os.path.join(path, "modes")
         os.makedirs(MODES, exist_ok=True)
         GEOMETRY = os.path.join(path, "geometry")
@@ -247,17 +283,18 @@ def setup(
 
         for run in runs:
             for k, v in run["sources"].items():
-                p = ports[k]
+                # p = ports[k]
+                p = next((p for p in ports if p["name"] == k), None)
                 ct = np.array(p["center"])
                 n = np.array(p["normal"])
                 v["center"] = (ct + n * source_port_margin).tolist()
 
                 # modes = solve_modes(polys, eps, bbox)
             for k, v in run["monitors"].items():
-                p = ports[k]
+                p = next((p for p in ports if p["name"] == k), None)
                 v["center"] = copy.deepcopy(p["center"])
             for k, v in list(run["sources"].items()) + list(run["monitors"].items()):
-                p = ports[k]
+                p = next((p for p in ports if p["name"] == k), None)
                 v["frame"] = p["frame"]
                 if len(v["center"]) == 2:
                     v["center"] += [zcenter]
@@ -271,7 +308,7 @@ def setup(
         hasPEC = False
         for f in os.listdir(GEOMETRY):
             i, mat, _ = f[:-4].split("_")
-            if mat != "design":
+            if mat != "design" and mat != "None":
                 if mat != "PEC":
                     eps = materials[mat]["epsilon"]
                 else:
@@ -281,24 +318,33 @@ def setup(
         mesheps = [x[1:] for x in sorted(ime, key=lambda x: x[0])]
         l = []
         for mode in modes:
-            v = runs[0]["monitors"][mode["ports"][0]]
+            if not mode["ports"]:
+                v = runs[0]["monitors"].values()[0]
+            else:
+                v = runs[0]["monitors"][mode["ports"][0]]
             start = v["start"]
             stop = v["stop"]
 
             L = stop[0] - start[0]
             W = stop[1] - start[1]
-            dl = L / 50
+            if hasPEC:
+                dl = mode.get("dx", L / 100)
+            else:
+                dl = mode.get("dx", L / 50)
             dx = L / round(L / dl)
             dy = W / round(W / dl)
 
             if "frequency" not in mode and "wavelength" not in mode:
-                mode["wavelength"] = center_wavelength
+                mode["wavelength"] = wavelength
             elif "frequency" in mode:
                 mode["wavelength"] = wl_1_f / mode["frequency"]
 
             if "modes" in mode:
                 mode["dl"] = [dx, dy]
-
+                for _mode in mode["modes"]:
+                    for k, v in _mode.items():
+                        if type(v) is np.ndarray:
+                            _mode[k] = v.tolist()
                 l.append(mode)
             else:
                 polyeps = section_mesh(
@@ -318,6 +364,7 @@ def setup(
                 nmodes = mode.get("nmodes", 1)
 
                 wavelength = mode["wavelength"]
+                eps = epsilon_from_polyeps(polyeps, bg, start, stop, dx, dy).tolist()
                 if hasPEC:
                     _modes = solvemodes_femwell(
                         polyeps,
@@ -336,10 +383,11 @@ def setup(
                     )
                 l.append(
                     {
-                        "wavelength": wavelength,
+                        "wavelength_range": mode["wavelength_range"],
                         "modes": _modes,
                         "ports": mode["ports"],
                         "dl": [dx, dy],
+                        "epsilon": eps,
                     }
                 )
         prob["modes"] = l
