@@ -16,6 +16,7 @@ def make_prob(
     path,
     component=None,
     wavelengths=None,
+    wavelength=None,
     entries=None,
     keys=["2,1"],
     study="sparams",
@@ -24,7 +25,7 @@ def make_prob(
     monitors=None,
     frequencies=None,
     wl_1_f=None,
-    modes=[],
+    modes=None,
     #
     layer_stack={},
     materials=MATERIALS,
@@ -33,15 +34,22 @@ def make_prob(
     targets=None,
     symmetries=[],
     # design_layer=DESIGN,
-    init_holes=True,
+    init_pattern_level=None,
+    init_pattern_spacing=None,
+    init_pattern_diameter=None,
+    init_grayscale=True,
+    area_change=0.03,
     fill_material=None,
     void_material=None,
     iters=None,
     lmin=None,
     stoploss=None,
+    design_ports=None,
+    #
     path_length_multiple=0.85,
     approx_2D_mode=None,
     pixel_size=0.01,
+    info=None,
     **kwargs
 ):
     if approx_2D_mode:
@@ -77,23 +85,22 @@ def make_prob(
         assert wavelengths is None
         wavelengths = [wl_1_f / f for f in frequencies]
 
-    for mode in modes:
-        if "frequency" in mode:
-            if "wavelength" not in mode:
-                mode["wavelength"] = wl_1_f / mode["frequency"]
-
     if type(wavelengths) in [int, float]:
         wavelengths = [wavelengths]
     wavelengths = sorted(wavelengths)
-    center_wavelength = median(wavelengths)
-    # nres *= center_wavelength / min(map(min, wavelengths))
+    if wavelength is None:
+        wavelength = median(wavelengths)
+    # nres *= wavelength / min(map(min, wavelengths))
 
     if not entries:
         entries = []
 
         for w in wavelengths:
             for k in keys:
-                entries.append([w, *unpack_sparam_key(k)])
+                try:
+                    entries.append([w, *unpack_sparam_key(k)])
+                except Exception as e:
+                    0
     l = []
     for w, po, mo, pi, mi in entries:
         k = [w, pi, mi, pi, mi]
@@ -144,7 +151,7 @@ def make_prob(
         path,
         component=component,
         study=study,
-        center_wavelength=center_wavelength,
+        wavelength=wavelength,
         wl_1_f=wl_1_f,
         wavelengths=wavelengths,
         runs=runs,
@@ -157,6 +164,7 @@ def make_prob(
         nres=nres,
         approx_2D_mode=approx_2D_mode,
         pixel_size=pixel_size,
+        info=info,
         #  sources=sources, monitors=monitors,
         **kwargs
     )
@@ -197,11 +205,16 @@ def make_prob(
             **{
                 "targets": targets,
                 "designs": designs,
-                "init_holes": init_holes,
+                "init_pattern_level": init_pattern_level,
+                "init_pattern_spacing": init_pattern_spacing,
+                "init_pattern_diameter": init_pattern_diameter,
+                "init_grayscale": init_grayscale,
+                "area_change": area_change,
                 "iters": iters,
                 "lmin": lmin,
                 "stoploss": stoploss,
                 "path_length_multiple": path_length_multiple,
+                "design_ports": design_ports,
             },
         }
     save_problem(prob, path)
