@@ -35,7 +35,7 @@ def start_fdtd_server(url=URL):
             print(err_message)
 
 
-def solve(path, **kwargs):
+def solve(path, nthreads=4, **kwargs):
     path = os.path.abspath(path)
     prob = load_prob(path)
     prob = {**prob, **kwargs}
@@ -61,6 +61,7 @@ def solve(path, **kwargs):
                 print(str(line.decode().strip()), flush=True)
             err_message = proc.stderr.read().decode()
             print(err_message)
+        return proc.returncode
 
     # if dev:
     #     env = r'using Pkg;Pkg.activate(raw"c:\Users\pxshe\OneDrive\Desktop\beans\Luminescent.jl\luminescent");'
@@ -68,7 +69,11 @@ def solve(path, **kwargs):
     #     env = '0;'
     # cmd = ["lumi", path]
     # try:
-    run(["Luminescent", path, " --julia-args -t4"])
+    c = run(["Luminescent", path, f" --julia-args -t{nthreads}"])
+    if c != 0:
+        print("failed")
+        exit(1)
+
     # except:
     # run(["julia", "-e", f'println(Base.active_project())'])
     # print("no binaries found - starting julia session to compile - will alternate between execution and JIT compilation - will take 3 mins before simulation starts.\nYou can take a break and come back :) ...")
@@ -104,8 +109,8 @@ def load_sparams(sparams):
     if "re" in list(sparams.values())[0]:
         return {k: v["re"] + 1j * v["im"] for k, v in sparams.items()}
     return {
-        center_wavelength: {k: (v["re"] + 1j * v["im"]) for k, v in d.items()}
-        for center_wavelength, d in sparams.items()
+        wavelength: {k: (v["re"] + 1j * v["im"]) for k, v in d.items()}
+        for wavelength, d in sparams.items()
     }
 
 
@@ -114,6 +119,8 @@ def load_sol(path, show=True):
     print(f"loading solution from {path}")
     prob = json.loads(open(os.path.join(path, "problem.json"), "rb").read())
     p = os.path.join(path, "solution.json")
+    # if os.path.exists(p):
+    #     sol = json.loads(open(p).read())
     sol = json.loads(open(p).read())
 
     if prob["class"] == "gen":
@@ -182,6 +189,16 @@ def load_sol(path, show=True):
         #         Video(fn)
         0
 
-    make_gds(path)
+    make_design_gds(path)
 
     return sol
+
+
+def design_from_gds(path, i=1):
+    # make_design_gds(path)
+    c = gf.import_gds(os.path.join(path, f"design{i}.gds"))
+    d = json.load(open(os.path.join(path, "info.json")))
+    for p in d[f"designs"][i - 1]["ports"]:
+        c.add_port(**p, layer=WG)
+    c.info = d
+    return c
