@@ -318,7 +318,7 @@ def atcentroid(a):
     return a
 
 
-def solvemodes(polyeps, background_eps, start, stop, wavelength, num_modes, dx, dy):
+def epsilon_from_polyeps(polyeps, background_eps, start, stop, dx, dy):
     L = stop[0] - start[0]
     W = stop[1] - start[1]
     m = int(L / dx)
@@ -326,22 +326,49 @@ def solvemodes(polyeps, background_eps, start, stop, wavelength, num_modes, dx,
 
     epsilon = np.zeros((m, n)) + background_eps
     polyeps.reverse()
+    for poly, eps in polyeps:
+        if eps != "PEC":
+            poly = shapely.affinity.translate(poly, xoff=-start[0], yoff=-start[1])
+            poly = shapely.affinity.scale(
+                poly, xfact=1 / dx, yfact=1 / dy, origin=(0, 0)
+            )
+            poly = clip_by_rect(poly, 0, 0, m, n)
+            mask = rasterio.features.rasterize([poly], out_shape=epsilon.T.shape)
+            mask = mask.T
+            epsilon *= 1 - mask
+            epsilon += mask * eps
+    polyeps.reverse()
+    return epsilon
+
+
+def solvemodes(polyeps, background_eps, start, stop, wavelength, num_modes, dx, dy):
+    L = stop[0] - start[0]
+    W = stop[1] - start[1]
+    m = int(L / dx)
+    n = int(W / dy)
+
+    epsilon = np.zeros((m - 1, n - 1)) + background_eps
+    polyeps.reverse()
     for poly, eps in polyeps:
         poly = shapely.affinity.translate(poly, xoff=-start[0], yoff=-start[1])
         poly = shapely.affinity.scale(poly, xfact=1 / dx, yfact=1 / dy, origin=(0, 0))
+        poly = clip_by_rect(poly, 0.5, 0.5, m - 0.5, n - 0.5)
         mask = rasterio.features.rasterize([poly], out_shape=epsilon.T.shape)
         mask = mask.T
         epsilon *= 1 - mask
         epsilon += mask * eps
+    polyeps.reverse()
 
-    m += 1
-    n += 1
-    x = np.linspace(0, L, m)
-    y = np.linspace(0, W, n)
+    # x = np.linspace(0, L, m)
+    # y = np.linspace(0, W, n)
+    x = np.linspace(dx / 2, L - dx / 2, m)
+    y = np.linspace(dy / 2, W - dy / 2, n)
     # plt.matshow(epsilon)
     # plt.colorbar()  # Add a colorbar to show the mapping
     # plt.show()
 
+    # print(start, stop)
+
     def ϵfunc(x_, y_):
         return epsilon
 
@@ -354,9 +381,15 @@ def solvemodes(polyeps, background_eps, start, stop, wavelength, num_modes, dx,
 
     neffs = [np.real(m.neff) for m in modes]
     modes = [
+        # {k: m.get_field(k, x, y).T for k in ["Ex", "Ey", "Ez", "Hx", "Hy", "Hz"]}
         {
-            k: atcentroid(m.get_field(k, x, y).T)
-            for k in ["Ex", "Ey", "Ez", "Hx", "Hy", "Hz"]
+            k: m.get_field(k, x, y).T
+            for k in [
+                "Ex",
+                "Ey",
+                "Hx",
+                "Hy",
+            ]
         }
         for m in modes
     ]
@@ -380,18 +413,19 @@ def solvemodes_femwell(
 ):
     xmin, ymin = start
     xmax, ymax = stop
+    L = xmax - xmin
+    W = ymax - ymin
     metallic_boundaries = []
     c = []
     i = 0
-    dr = dx / 4
     for x in PEC_boundaries:
         if x == "x+":
             p = shapely.LineString([(xmax, ymin), (xmax, ymax)])
         elif x == "x-":
             p = shapely.LineString([(xmin, ymin), (xmin, ymax)])
-        elif x == "y+":
+        elif x == "+y":
             p = shapely.LineString([(xmin, ymax), (xmax, ymax)])
-        elif x == "y-":
+        elif x == "-y":
             p = shapely.LineString([(xmin, ymin), (xmax, ymin)])
         else:
             raise ValueError(f"Unknown PEC boundary {x}")
@@ -420,7 +454,7 @@ def solvemodes_femwell(
 
     resolutions = {
         k: {
-            "resolution": (wavelength / 10 / np.sqrt(epsilons[k])),
+            "resolution": dx,
             "distance": wavelength,
         }
         for k in polygons
@@ -455,8 +489,12 @@ def solvemodes_femwell(
     basis_fix = basis.with_element(ElementVector(ElementDG(ElementTriP1())))
 
     grid_x, grid_y = np.meshgrid(
-        np.linspace(xmin + dx / 2, xmax - dx / 2, round((xmax - xmin) / dx)),
-        np.linspace(ymin + dy / 2, ymax - dy / 2, round((ymax - ymin) / dy)),
+        np.linspace(
+            xmin + dx / 2, xmax - dx / 2, round((xmax - xmin) / dx), dtype=np.float32
+        ),
+        np.linspace(
+            ymin + dy / 2, ymax - dy / 2, round((ymax - ymin) / dy), dtype=np.float32
+        ),
     )
     coordinates = np.array([grid_x.flatten(), grid_y.flatten()])
     shape = grid_x.shape
@@ -487,15 +525,15 @@ def solvemodes_femwell(
             )
         ).T
 
-        # plt.imshow(E[:, 0].reshape(shape).real)
-        # plt.show()
+        plt.imshow(E[:, 1].reshape(shape).real)
+        plt.show()
         d = {
             "Ex": E[:, 0],
             "Ey": E[:, 1],
-            # "Ez": E[:, 2],
+            "Ez": E[:, 2],
             "Hx": H[:, 0],
             "Hy": H[:, 1],
-            # "Hz": H[:, 2],
+            "Hz": H[:, 2],
         }
         d = {k: v.reshape(shape) for k, v in d.items()}
         d = {
@@ -603,26 +641,30 @@ def create_gif(image_path, output_path, duration):
     )
 
 
-def make_movie(path, fps=5):
-    FRAMES = os.path.join(path, "frames")
-    MOVIE = os.path.join(path, "simulation.mp4")
+def make_movie(path, fps=5, frames_folder=None, video_name=None):
     # shutil.rmtree(MOVIE, ignore_errors=True)
     # GIF = os.path.join(path, "sim.gif")
     # create_gif(FRAMES, GIF, duration)
     # return Image(open(GIF, "rb").read())
-    image_folder = FRAMES
-    video_name = MOVIE
+    if frames_folder is None:
+        frames_folder = os.path.join(path, "frames")
+    if video_name is None:
+        video_name = os.path.join(path, "simulation.mp4")
 
     images = [
         img
-        for img in os.listdir(image_folder)
+        for img in os.listdir(frames_folder)
         if img.endswith((".jpg", ".jpeg", ".png"))
     ]
-    images = sorted(images, key=lambda x: float(x.split(".")[0]))
+    try:
+        images = sorted(images, key=lambda x: float(x.split(".")[0]))
+    except:
+        images = sorted(images)
+
     # print("Images:", images)
 
     # Set frame from the first image
-    frame = cv2.imread(os.path.join(image_folder, images[0]))
+    frame = cv2.imread(os.path.join(frames_folder, images[0]))
     height, width, layers = frame.shape
 
     # Video writer to create .avi file
@@ -632,7 +674,7 @@ def make_movie(path, fps=5):
 
     # Appending images to video
     for image in images:
-        video.write(cv2.imread(os.path.join(image_folder, image)))
+        video.write(cv2.imread(os.path.join(frames_folder, image)))
 
     # Release the video file
     video.release()
@@ -640,19 +682,30 @@ def make_movie(path, fps=5):
     print("Video generated successfully!")
 
 
-def make_gds(path, prob=None):
+def make_top_movie(path, **kwargs):
+    FRAMES = os.path.join(path, "temp", "frames")
+    os.makedirs(FRAMES, exist_ok=True)
+    for d in os.listdir(os.path.join(path, "checkpoints")):
+        shutil.copy(
+            os.path.join(path, "checkpoints", d, "snapshot.png"),
+            os.path.join(FRAMES, f"{d}_snapshot.png"),
+        )
+    make_movie(path, frames_folder=FRAMES, video_name="opt.mp4", **kwargs)
+
+
+def make_design_gds(path, prob=None):
     if prob is None:
         prob = load_prob(path)
     for fn in os.listdir(path):
         if fn.endswith(".npy") and fn.startswith("design"):
             a = np.load(os.path.join(path, fn))
-            c = gf.read.from_np(a, prob["pixel_size"] * 1e3)
+            c = gf.read.from_np(a, prob["pixel_size"] * 1e3, threshold=0.5)
             c.write_gds(os.path.join(path, f"{fn[:-4]}.gds"))
 
             a = np.uint8(a) * 255  # Convert to uint8
             image = PILImage.fromarray(a)  # Convert to uint8 if needed
             image.save(os.path.join(path, f"{fn[:-4]}.png"))
-    CKPT = os.path.join(path, "checkpoints")
-    if os.path.exists(CKPT):
-        for d in os.listdir(CKPT):
-            make_gds(os.path.join(CKPT, d), prob)
+    # CKPT = os.path.join(path, "checkpoints")
+    # if os.path.exists(CKPT):
+    #     for d in os.listdir(CKPT):
+    #         make_design_gds(os.path.join(CKPT, d), prob)
